/*
 * rpc_ipc.c
 */

/*
 * Copyright (C) 2010-2011 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 
/************************************************************************
*  This file implements receiving remote procedure calls from the GPP   *
*  and calling the appropriate local DSP function and returning results *
************************************************************************/

/*  ----------------------------------- Standard headers            */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>  
#include <stdbool.h>

/*  ----------------------------------- XDC Runtime Headers         */
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Log.h>

/*  ----------------------------------- SysBios Headers             */
#include <ti/sysbios/knl/Task.h>

/*  ----------------------------------- SysLink IPC                 */
#include <ti/ipc/Ipc.h>
#include <ti/ipc/MultiProc.h>
#include <ti/ipc/HeapMemMP.h>
#include <ti/ipc/SharedRegion.h>
#include <ti/ipc/MessageQ.h>
#include <ti/ipc/Notify.h>

/*  ----------------------------------- C6Run common Header         */
#include "C6Run_common.h"

/*  ----------------------------------- This module's header        */
#include "rpc_ipc.h"


/************************************************************
* Explicit External Declarations                            *
************************************************************/


/************************************************************
* Local Macro Declarations                                  *
************************************************************/


/************************************************************
* Local Typedef Declarations                                *
************************************************************/


/************************************************************
* Local Function Declarations                               *
************************************************************/


/************************************************************
* Local Variable Definitions                                *
************************************************************/

static bool LOCAL_isModuleInitialized = false;


/************************************************************
* Global Variable Definitions                               *
************************************************************/

MessageQ_Handle   MessageQ_rpcFromGPP;
MessageQ_QueueId  MessageQ_rpcToGPPId;

RPC_Msg *RPC_msg;


/************************************************************
* Global Function Definitions                               *
************************************************************/

int32_t RPC_IPC_init( void )
{
  int32_t status = C6RUN_SUCCESS;
  
  // If already initialized, just exit
  if (LOCAL_isModuleInitialized)
  {
    return status;
  }

  // Create a message queue instance
  MessageQ_rpcFromGPP = MessageQ_create(RPC_FROM_GPP_MSGQNAME, NULL);
  if (MessageQ_rpcFromGPP == NULL)
  {
    Log_error0("MessageQ_create failed\n" );
    return C6RUN_FAIL;
  }

  // Open the remote message queue. Spin until it is ready.
  do
  {
    status = MessageQ_open(RPC_TO_GPP_MSGQNAME, &MessageQ_rpcToGPPId);
    if (status == MessageQ_E_NOTFOUND)
    {
      // Sleep for a while before trying again.
      Task_sleep (10);
    }
  }
  while (status != MessageQ_S_SUCCESS);
  
  LOCAL_isModuleInitialized = true;
  
  return status;
}

void RPC_IPC_exit( void )
{
  int32_t status;
  
  // If not already initialized, just exit
  if (!LOCAL_isModuleInitialized)
  {
    return;
  }  

  // Disconnect from the remote queue on the GPP
  status = MessageQ_close (&MessageQ_rpcToGPPId);
  if (status < 0)
  {
    Log_error0("MessageQ_close failed\n");
  }
  
  // Delete the local receive queue
  status = MessageQ_delete (&MessageQ_rpcFromGPP);
  if (status < 0)
  {
    Log_error0("MessageQ_delete failed\n");
  }
  

  LOCAL_isModuleInitialized = false;
}

// RPC_IPC_sendMsg - Sends the data and parameters on to the GPP
void RPC_IPC_sendMsg( RPC_Msg *msg, uint16_t msgId )
{
  int32_t status;

  // Setup message and send  
  MessageQ_setMsgId((MessageQ_Msg) msg, msgId);
  MessageQ_setReplyQueue(MessageQ_rpcFromGPP,(MessageQ_Msg) msg);

  status = MessageQ_put(MessageQ_rpcToGPPId, (MessageQ_Msg) msg);
  
  if (status < 0)
  { 
    Log_error1("MessageQ_put() failed: status = [0x%x]\n", status);
  }
}

// RPC_IPC_recvMsg - Reads the data and parameters passed from the GPP
uint16_t RPC_IPC_recvMsg( RPC_Msg **msg )
{
  int32_t status;
  uint16_t msgId;

  status = MessageQ_get(MessageQ_rpcFromGPP, (MessageQ_Msg *) msg, MessageQ_FOREVER);

  if (status < 0)
  {
    Log_error1("MessageQ_get() failed: status = [0x%x]\n", status); 
    return RPC_NULL;
  }

  msgId = MessageQ_getMsgId((MessageQ_Msg) (*msg));
  
  return msgId;
}


/***********************************************************
* Local Function Definitions                               *
***********************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/
