/*
 * control_ipc.c
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/************************************************************************
*  Routines for handling C I/O requests from the DSP                    *
************************************************************************/

/*  ----------------------------------- Standard Headers                */
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

/*  ----------------------------------- Linux Posix Headers            */
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>

/*  ----------------------------------- DSP/BIOS Link                   */
#include <dsplink.h>

/*  ----------------------------------- DSP/BIOS LINK API               */
#include <proc.h>
#include <procdefs.h>
#include <msgq.h>
#include <pool.h>
#include <loaderdefs.h>

/*  ----------------------------------- CMEM API                        */
#include <ti/sdo/linuxutils/cmem/include/cmem.h>

#if defined(LPM_REQUIRED)
  /*  --------------------------------- LPM API                         */
  #include <ti/bios/power/lpm.h>
#endif

/*  ----------------------------------- Application headers             */
#include "debug_common.h"
#include "C6Run_common.h"
#include "C6Run.h"
#include "C6Run_mem.h"
#include "cio_ipc.h"
#include "rpc_ipc.h"

/*  ----------------------------------- This module's header            */
#include "control_ipc.h"


/************************************************************
* Explicit External Declarations                            *
************************************************************/

// Extern declaration to the default DSP/BIOS LINK configuration structure.
extern LINKCFG_Object LINKCFG_config;

// Platform specific DSP Object for Link configuration
extern LINKCFG_Dsp myDspObject;

// Platform specific memory table pointer for updating DSP memory map
// in Link configuration
extern LINKCFG_MemEntry* myMemTables[];

extern uint8_t C6RUN_IPC_processorId;


/************************************************************
* Local Macro Declarations                                  *
************************************************************/


/************************************************************
* Local Typedef Declarations                                *
************************************************************/


/************************************************************
* Local Function Declarations                               *
************************************************************/


/************************************************************
* Local Variable Definitions                                *
************************************************************/


/************************************************************
* Global Variable Definitions                               *
************************************************************/

volatile bool CONTROL_IPC_initialized  = FALSE;

// Global CONTROL MSGQ Objects for Tx and Rx
MSGQ_Queue MSGQ_controlFromGPP = (uint32_t) MSGQ_INVALIDMSGQ;
MSGQ_Queue MSGQ_controlToGPP = (uint32_t) MSGQ_INVALIDMSGQ;

CONTROL_Msg *CONTROL_msg;


/************************************************************
* Global Function Definitions                               *
************************************************************/

int CONTROL_IPC_init( void )
{
  DSP_STATUS status = DSP_SOK;

  if (CONTROL_IPC_initialized)
  {
    return C6RUN_SUCCESS;
  }
  
  VERBOSE_PRINT0("Entered CONTROL_IPC_init()\n");
    
  // Open the CONTROL TO GPP message queue
  if (DSP_SUCCEEDED(status))
  {
    status = MSGQ_open(CONTROL_TO_GPP_MSGQNAME, &MSGQ_controlToGPP, NULL);
    if (DSP_FAILED(status))
    {
      REPORT_STATUS("MSGQ_open");
      return C6RUN_FAIL;
    }
  }
  
  // Locate the CONTROL FROM GPP message queue
  if (DSP_SUCCEEDED(status))
  {
    MSGQ_LocateAttrs syncLocateAttrs;
  
    syncLocateAttrs.timeout = WAIT_FOREVER;
    status = DSP_ENOTFOUND;
    while ((status == DSP_ENOTFOUND) || (status == DSP_ENOTREADY))
    {
      status = MSGQ_locate(CONTROL_FROM_GPP_MSGQNAME, &MSGQ_controlFromGPP, &syncLocateAttrs);
      if ((status == DSP_ENOTFOUND) || (status == DSP_ENOTREADY))
      { 
        VERBOSE_PRINT0("CONTROL MSGQ_locate() failed! Will retry.\n");
        usleep(10000);
      }
      else if (DSP_FAILED(status))
      {
        REPORT_STATUS("MSGQ_locate");
        return C6RUN_FAIL;
      }
    }
  }
  
  // Set the message queue that will receive any async. errors
  if (DSP_SUCCEEDED(status))
  {
    status = MSGQ_setErrorHandler(MSGQ_controlToGPP, POOL_makePoolId(C6RUN_IPC_processorId, COMMON_POOL_ID));
    if (DSP_FAILED(status))
    {
      printf("MSGQ_setErrorHandler () failed. Status = [0x%x]\n", (uint32_t)status);
      return C6RUN_FAIL;
    }
  }
  VERBOSE_PRINT0("MSGQ_setErrorHandler() passed!\n");
  
  // Do initial MSGQ alloc for control (since we will start sending from the GPP side)
  status = MSGQ_alloc(COMMON_POOL_ID, sizeof(CONTROL_Msg), (MSGQ_Msg *) &CONTROL_msg );
  if (DSP_FAILED(status))
  { 
    REPORT_STATUS("MSGQ_alloc");
    return C6RUN_FAIL;
  }
  
  CONTROL_IPC_initialized = TRUE;
  
  VERBOSE_PRINT0("Leaving CONTROL_IPC_init()\n");
  
  return C6RUN_SUCCESS;
}

void CONTROL_IPC_exit()
{
  DSP_STATUS status = DSP_SOK;
  DSP_STATUS tmpStatus = DSP_SOK;
  
  if (!CONTROL_IPC_initialized)
  {
    return;
  }

  VERBOSE_PRINT0("Entered CONTROL_IPC_exit()\n");

  // Free the control message since we are not sending it back
  MSGQ_free((MsgqMsg ) CONTROL_msg);
  
  // Release the remote message queue
  status = MSGQ_release(MSGQ_controlFromGPP);
  if (DSP_FAILED(status))
  {
    printf("MSGQ_release() failed. Status = [0x%x]\n", (uint32_t)status);
  }
  else
  {
    VERBOSE_PRINT0("MSGQ_release() passed.\n");
  }

  // Close the GPP's CONTROL message queue
  tmpStatus = MSGQ_close(MSGQ_controlToGPP);
  if (DSP_SUCCEEDED(status) && DSP_FAILED(tmpStatus))
  {
    status = tmpStatus;
    REPORT_STATUS("MSGQ_close");
  } 
  else
  {
    VERBOSE_PRINT0("MSGQ_close() passed.\n");
  }

  CONTROL_IPC_initialized = FALSE;    

  VERBOSE_PRINT0("Leaving CONTROL_IPC_exit()\n");
}

void CONTROL_IPC_send(uint16_t command)
{
  DSP_STATUS status;
  
  VERBOSE_PRINT0("Entering CONTROL_IPC_send()\n");

  MSGQ_setMsgId((MsgqMsg)CONTROL_msg, command);

  status = MSGQ_put(MSGQ_controlFromGPP, (MsgqMsg) CONTROL_msg);
  
  if (DSP_FAILED(status))
  { 
    MSGQ_free((MsgqMsg) CONTROL_msg); 
    REPORT_STATUS("MSGQ_put"); 
  }
  
  VERBOSE_PRINT0("Exiting CONTROL_IPC_send()\n");
}

uint16_t CONTROL_IPC_recv()
{
  DSP_STATUS status;
  uint16_t msgId;
  
  VERBOSE_PRINT0("Entering CONTROL_IPC_recv()\n");

  status = MSGQ_get(MSGQ_controlToGPP, WAIT_FOREVER, (MsgqMsg *) &CONTROL_msg);

  if (DSP_FAILED(status))
  {
    REPORT_STATUS("MSGQ_get"); 
    return CONTROL_NULL;
  }

  msgId = MSGQ_getMsgId(CONTROL_msg);

  VERBOSE_PRINT0("Exiting CONTROL_IPC_recv()\n");
  
  return msgId;
}


/***********************************************************
* Local Function Definitions                               *
***********************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/

